/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

//
// Traits classes that map container types and constness tags to iterator types.
//

#if!defined(_VIEW_BASE_TRAITS_HPP)
#define _VIEW_BASE_TRAITS_HPP

#if !defined(_VIEW_CONFIG_HPP)
#include "view_config.hpp"
#endif

#if !defined(__SGI_STL_INTERNAL_RELOPS)
#include "view_relops.hpp"
#endif

#if !defined(_SYS_LIBRARY_ITERATOR_)
#include <iterator>
#define _SYS_LIBRARY_ITERATOR_
#endif


BEGIN_VIEW_NAMESPACE
//BEGIN_VIEW_DETAILS_NAMESPACE

//
// Traits classes mapping (container,const_tag) pairs to value
// and iterator types.
//
template<class container, class const_tag>
struct view_traits {
  typedef container container_type;
  typedef typename container::iterator iterator;
  typedef typename container::const_iterator const_iterator;
  typedef std::iterator_traits<iterator>::value_type value_type;
};


template <class container>
struct view_traits<container,const_view_tag> {
  typedef const container container_type;
  typedef typename container::const_iterator iterator;
  typedef typename container::const_iterator const_iterator;
  // I'm not sure if iterator_traits<T const*>::value_type == T as
  // provided by the SGI STL as of 1999 is standards conform or not.
  // At least it's a violation of the requirement that for every
  // iterator i the expression *i must be convertible to the value type.
  typedef std::iterator_traits<iterator>::value_type const value_type;
};

template<class container, class const_tag>
struct reversible_view_traits: public view_traits<container,const_tag> {
  typedef typename container::reverse_iterator reverse_iterator;
  typedef typename container::const_reverse_iterator const_reverse_iterator;
};

template<class container>
struct reversible_view_traits<container,const_view_tag>
  : public view_traits<container,const_view_tag> {
  typedef typename container::const_reverse_iterator reverse_iterator;
  typedef typename container::const_reverse_iterator const_reverse_iterator;
};

//END_VIEW_DETAILS_NAMESPACE
END_VIEW_NAMESPACE

#endif // _VIEW_BASE_TRAITS_HPP
// $Id: view_base_traits.hpp,v 1.1 2000/11/20 14:11:14 bzfweise Exp $
