/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#if !defined(_MAP_VIEW_FWD_HPP)
#define _MAP_VIEW_FWD_HPP

#if !defined(_VIEW_CONFIG_HPP)
#include "details/view_config.hpp"
#endif

#if !defined(__SGI_STL_INTERNAL_RELOPS)
#include "view_relops.hpp"
#endif

#if !defined(_VIEW_BASE_HPP)
#include "view_base.hpp"
#endif

#if !defined(_VIEW_OWNERSHIP_HPP)
#include "view_ownership.hpp"
#endif

#if !defined(_SYS_LIBRARY_ITERATOR_)
#include <iterator>
#define _SYS_LIBRARY_ITERATOR_
#endif

// A forward declaration of every public view.
// The purpose is that if you have an include file that has a pointer to a view in
// it you don't need to include the full view.hpp file. Instead you can include just this
// file. Then the .cpp file can include the full view.hpp file and then use it.
// For each view, I have included the view and the iterator declaration if necessary.

BEGIN_VIEW_NAMESPACE

template <class container,
          class const_tag = const_view_tag,
          class ownership_tag = external_ownership>
class map_keys;

template <class container,
          class predicate, 
          class const_tag = const_view_tag,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_keys_filtered;

template <class container,
          class transform,
          class const_tag = const_view_tag,
          class T=typename transform::result_type,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_keys_transformed;


template <class container,
          class predicate,
          class transform,
          class const_tag = const_view_tag, 
          class T=typename transform::result_type,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_keys_view;

template <class container,
          class const_tag = const_view_tag,
          class ownership_tag = external_ownership>
class map_values;

template <class container,
          class predicate, 
          class const_tag = const_view_tag,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_values_filtered;

template <class container,
          class transform,
          class const_tag = const_view_tag,
          class T=typename transform::result_type,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_values_transformed;

template <class container,
          class predicate,
          class transform,
          class const_tag = const_view_tag, 
          class T=typename transform::result_type,
          class iterator_tag = typename std::iterator_traits<typename container::iterator>::iterator_category,
          class ownership_tag = external_ownership>
class map_values_view;


END_VIEW_NAMESPACE

#endif // _MAP_VIEW_FWD_HPP
