/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999,2000  Gary Powell
 * Copyright (c) 1999,2000  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#if !defined (_SORTED_SYM_DIFF_VIEW_H)
#define _SORTED_SYM_DIFF_VIEW_H	1

#if !defined(_SORTED_SYM_DIFF_VIEW_FWD_HPP)
#include "sorted_sym_diff_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined(_SORTED_DIFFERENCE_VIEW_HPP)
#include "sorted_difference_view.hpp"
#endif

#if !defined(_MERGE_VIEW_HPP)
#include "merge_view.hpp"
#endif

BEGIN_VIEW_NAMESPACE
BEGIN_VIEW_DETAILS_NAMESPACE

// definition of the base, but no implementation,
// just the two specializations.
template<class container_a, class container_b,
	  class comparator,
	  class const_tag,
	  class iterator_tag,
	  class ownership_tag_a,
	  class ownership_tag_b
	  >
// Not really a namespace but that's all I'm using it for.
class sorted_symmetrical_difference_namespace
{
private:
  typedef typename view_traits<container_a,const_tag>::container_type domain_a;
  typedef typename view_traits<container_b,const_tag>::container_type domain_b;
  // Define a filter_view of container_a which excludes any members
  // found in b.
  typedef sorted_difference_view<domain_a,domain_b,
            comparator,const_tag,iterator_tag,ownership_tag_a, ownership_tag_b> aWithoutb_view;

  // Define a filter_view of container_b which excludes any members
  // found in a.
  typedef sorted_difference_view<domain_b, domain_a,
            comparator, const_tag,iterator_tag,ownership_tag_b, ownership_tag_a> bWithouta_view;

  // Define a merge_view of the two filtered views.
  // Since each filtered view excludes the members found in the other
  // view, this will be a symmetric_difference_view.
  typedef merge_view<aWithoutb_view,bWithouta_view,
                     comparator,const_tag,iterator_tag,aggregated_ownership,aggregated_ownership> base_view;
protected:
  // There is this one more layer so we don't have to export out
  // all of the typedefs we set up in symmetric_difference_namespace.
  class view : public base_view
  {
  public:
    view(domain_a &a, domain_b &b, comparator const &c)
      : base_view(aWithoutb_view(a,b,c),
                  bWithouta_view(b,a,c))
    {}
  };
};
END_VIEW_DETAILS_NAMESPACE


//
// A sorted_symmetrical_difference_view.
//
// A view with two sorted containers, where an iterator will
// return the next in order item which is in only one of the containers.
//
//
template <class container_a, class container_b,
	  class comparator,
	  class const_tag,
	  class iterator_tag,
	  class ownership_tag_a,
	  class ownership_tag_b>
class sorted_symmetrical_difference_view
  : public VIEW_DETAILS_NAMESPACE::sorted_symmetrical_difference_namespace<container_a,container_b,
				comparator,const_tag,iterator_tag,ownership_tag_a,ownership_tag_b>::view
{
public:
  typedef VIEW_DETAILS_NAMESPACE::sorted_symmetrical_difference_namespace<container_a,container_b,
					comparator,const_tag,iterator_tag,ownership_tag_a,ownership_tag_b>::view inherited;
  typedef ctor_arg<container_a,const_tag,ownership_tag_a>::type ctor_arg_type_a;
  typedef ctor_arg<container_b,const_tag,ownership_tag_b>::type ctor_arg_type_b;

  explicit sorted_symmetrical_difference_view() {}
  sorted_symmetrical_difference_view(ctor_arg_type_a& cont_a, ctor_arg_type_b& cont_b,
                                     comparator const &comp = comparator())
    : inherited(cont_a, cont_b, comp) 
  {}
};

END_VIEW_NAMESPACE
#endif	// _SORTED_SYM_DIFF_VIEW_H
